function [x,fx,fcount] = SA_HPS(n,U,L,f,dsn1,dsn2)
% *******************************************************
% SA_HPS = Simulated Annealing Heuristic Pattern Search
% The DSSA Code is created on Dec. 16, 2002 by A. Hedar 
% Reference: 
%       A. Hedar and M. Fukushima, Heuristic pattern search and its 
%       hybridization with simulated annealing for nonlinear global 
%       optimization, Optimization Methods and Software, 19 (2004) 291-308.
% *******************************************************
% *******************************************************
% Inputs:
%  n    = Number of the objective variables   
%  U    = n-Dimension Vector of upper limits of the objective variables
%  L    = n-Dimension Vector of lower limits of the objective variables
%  f    = Objective function
%  dsn1 = PS pruned direction set in case of getting a descent direction
%  dsn2 = PS pruned direction set in case of not getting a descent direction
%
% Outputs:
%  x      = Best point obtained by SAHPS 
%  fx     = Best function value obtained by SAHPS
%  fcount = Number of function evaluations                   
%   
% *******************************************************
% Initial Parameters
% *******************************************************
if nargin < 5; dsn1 = 3; dsn2 = 1; end
fcount = 0;             % Function evals. counter  
itc=0;	        		% number of iterations
maxi=50*n;
% *******************************************************
% Generating the initial trial
% ****************************
x=zeros(n,1);
for i=1:n;
    x(i)=L(i)+rand*(U(i)-L(i));
end
fx = feval(f,x);
fcount=fcount+1;
% *******************************************************
% HPS Parameters
% **************
eps = 0.001;            % Exploring points neighborhood radius
alpha = 0.001;          % Step size along the ADD
delta = 0.1*min(U-L);   % Mesh step size
n_PS = n;               % number of PS trails per iteration 
nP = n_PS; 
beta=1/sqrt(n);
% *******************************************************
% SA Parameters
% *************
rd = 0.9;		        % reduction factor of temp
maxrd_its=2*n;      	% number of iterations before reducte the temp
tol = 1.0D-8;           % termanation acurcy
n_SA= n;               % number of SA trails per iteration
nS = n_SA;
redus = 0.2*min(U-L);   % Reduis of generating new SA trial
% Estimating the T-paramters
y=zeros(n,1);
for i=1:n;
    sn=(-1)^(round(rand));
    y(i)=x(i)+sn*(0.5+0.5*rand)*redus;
end 
fy = feval(f,y);
fcount=fcount+1;
df =abs(fy-fx);
T=max(0,df/log(1/0.9));
Tmin=min(0.001,0.001*T);
if (fy < fx); x = y; fx = fy; end
xbest=x; fbest=fx;
% *******************************************************
% SA-HPS Main Loop
% *******************************************************
while (itc < maxi & T > Tmin & df > tol)
    rd_itc=0;
    while(rd_itc < maxrd_its)
        delta = 0.1*min(U-L);
        xold = x;
        fold = fx;
        % ***********************************************
        % SA Loop
        % ***********************************************
        accept = 0;
        SAcount = 0;
        for j=1:n_SA
            y=zeros(n,1);           
% Single point generation
% -----------------------
%             for i=1:n;
%                 sn=(-1)^(round(rand));
%                 y(i)=x(i)+sn*(0.2+2*rand)*redus;
%             end 
% =======================================================
% Exploring point generation
% --------------------------
            ye=zeros(n,1);
            for i=1:n;
                sn=(-1)^(round(rand));
                ye(i)=x(i)+0.01*sn*rand*eps;
            end                    
            fye = feval(f,ye);
            fcount=fcount+1;
            if (fye < fx)
                y=x+(0.01+1.9*rand)*redus*(ye-x)/norm(ye-x);
            else
                y=x-(0.01+1.9*rand)*redus*(ye-x)/norm(ye-x);
            end
% ========================================================
% ADD point generation
% --------------------
%             [v1,fc1] = dscnt_dir(f,x,n,fx,eps);
%             fcount=fcount+fc1;
%             y=x+(0.1+rand)*redus*(v1)/norm(v1);            
% ========================================================
            fy = feval(f,y);
            fcount=fcount+1;
            if (fy < fx)
                accept = accept+1;
                x = y; fx = fy;
                if (fx < fbest); xbest=x; fbest=fx; end
            else
                prob=exp(-(fy-fx)/T);
                if (prob >= rand)
                    accept = accept+1;
                    x = y; fx = fy; 
                end
            end
            SAcount=SAcount+1;
            itc=itc+1;      
        end
        % ***********************************************
        % PS Loop
        % ***********************************************
        if (accept <= 1)
            for k=1:n_PS        	    
				[v,fcount_ADD] = dscnt_dir(f,x,n,fx,eps);
                fcount=fcount+fcount_ADD;		
                xvd = x + delta*v/norm(v);
                fxvd = feval(f,xvd);
                fcount=fcount+1;
                if (fxvd < fx)
                    xb = xvd;
                    fb = fxvd;                
                else
                    xva = x+alpha*v/norm(v);
                    fxva = feval(f,xva);
                    fcount=fcount+1;
                    if (fxva <= fx)                         
                        D = PSDirctionSet1(n,v,beta,dsn1);
                        nd= size(D,2);
                        Px= zeros(n,nd);
                        fPx= zeros(nd,1);
                        for j=1:nd;
                            for i=1:n
                                Px(i,j)=x(i)+delta*D(i,j);
                            end
                            fPx(j)=feval(f,Px(:,j));
                        end
                        fcount=fcount+nd;
                        [fPxs,is]=sort(fPx); 
                        Pxtmp=Px(:,is); Px=Pxtmp; 
                        ftmp=fPx(is); fPx=ftmp;  
                        xb = Px(:,1);
                        fb = fPx(1);
                    else                       
                        D = PSDirctionSet2(n,v,beta,dsn2);
                        nd= size(D,2);
                        Px= zeros(n,nd);
                        fPx= zeros(nd,1);
                        for j=1:nd;
                            for i=1:n
                                Px(i,j)=x(i)+delta*D(i,j);
                            end
                            fPx(j)=feval(f,Px(:,j));
                        end                        
                        fcount=fcount+nd;        
                        [fPxs,is]=sort(fPx); 
                        Pxtmp=Px(:,is); Px=Pxtmp; 
                        ftmp=fPx(is); fPx=ftmp;  
                        xb = Px(:,1);
                        fb = fPx(1);
                    end  
                end
                if (fb < fx);
                    % Case #1: Getting an improvement point
                    % delta=1.0*delta;
                    x = xb; fx = fb;
                    if (fx < fbest); xbest=x; fbest=fx; end  
                else
                    % Case #2: Getting no improvement point                 
                    delta=0.7*delta;  
                    eps = max(0.00001,0.9*eps);
                end                
                itc=itc+1;      
            end            
        end	
        rd_itc = rd_itc+SAcount;
        if (x ~= xold); df = abs(fx-fold);end            
    end
    % *******************************************************
    % Parameter Update
    % *******************************************************
    redus = max(0.01*min(U-L),0.95*redus);
%     nS = max(2*n,0.99*nS);
%     n_SA = round(nS);
    nP = min(5*n,1.05*nP);
    n_PS = round(nP);
    T=T*rd;
end 
% *******************************************************
% Output Data
% *******************************************************
x=xbest;
fx=fbest;
[x1,FCoun,f1]=NeMe(n,f,x);
% [x1,f1,FCoun] = HPSm(x,f,3,1);
if f1 < fx
    x=x1(:,1);
    fx=f1;   
end
% *************************************************************************
